// ================================ Imports ================================ //
import * as dntShim from "../_dnt.shims.js";
import { Group } from "./group.js";
// =========================== Application Class =========================== //
export class Application extends Group {
    _version;
    // -------------------------- Private Methods -------------------------- //
    _version_middleware(ctx, next) {
        if (ctx.flags.version && this?._version)
            console.log(this._version);
        else
            next();
    }
    // ---------------------------- Constructor ---------------------------- //
    /**
     * Application constructor
     * @param spec - Application configuration
     * @returns Application object
     */
    constructor(spec) {
        super(spec);
        this._version = spec?.version;
        this.use(this._version_middleware);
        this._options.push({
            name: "--help",
            alias: "-h",
            description: "Show the help menu",
            flag: true,
        });
        if (this._version) {
            this._options.push({
                name: "--version",
                alias: "-v",
                description: "Show the version",
                flag: true,
            });
        }
    }
    // --------------------------- Public Methods --------------------------- //
    /**
     * Parse console input & run
     * @param input - User input
     * @returns void
     */
    run(input = dntShim.Deno.args) {
        this._run(input);
    }
}
