// =========================== Utility Functions =========================== //
export function deepFreeze(object) {
    for (const key in object) {
        if (!object[key] && Object.isFrozen(object[key]))
            continue;
        if (typeof object[key] !== "object" && typeof object[key] !== "function")
            continue;
        deepFreeze(object[key]);
    }
    return Object.freeze(object);
}
export function define(target, key, value) {
    if (key === "__proto__")
        return;
    Object.defineProperty(target, key, {
        value: value,
        enumerable: true,
        writable: true,
        configurable: true,
    });
}
export function matchAll(pattern, string) {
    if (!pattern.global)
        pattern = new RegExp(pattern, ["g", pattern.flags].join(""));
    if (string.length <= 0)
        return [];
    const matches = [];
    let match;
    do {
        match = pattern.exec(string);
        if (match)
            matches.push([...match]);
    } while (match);
    return matches;
}
export function maxLength(array) {
    let maxLength = 0;
    array.forEach((s) => {
        if (s.length > maxLength)
            maxLength = s.length;
    });
    return maxLength;
}
export function padStringTo(string, length, left) {
    if (length <= string.length)
        return string;
    const whitespace = Array(length - string.length)
        .fill(" ")
        .join("");
    const padded = left ? whitespace + string : string + whitespace;
    return padded;
}
export function removeFromArray(array, searchPattern) {
    const newArray = [];
    for (const s of array) {
        // RegExp.test() is broken here for some reason
        if (s.match(searchPattern))
            continue;
        newArray.push(s);
    }
    return newArray;
}
export function camelCase(words) {
    if (words.length < 1)
        return "";
    if (words.length === 1)
        return words[0];
    for (let i = 1; i < words.length; i++) {
        const word = words[i];
        const firstLetter = word.slice(0, 1).toUpperCase();
        words[i] = `${firstLetter}${word.substring(1)}`;
    }
    return words.join("");
}
