"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const dntShim = __importStar(require("../_dnt.shims.js"));
const regexes_js_1 = __importDefault(require("./regexes.js"));
const parsing_js_1 = require("./parsing.js");
const utilities_js_1 = require("./utilities.js");
const logging_js_1 = require("./logging.js");
// ============================== Group Class ============================== //
class Group {
    _groups;
    _options;
    _commands;
    _description;
    _middlewares;
    // ------------------------- Protected Methods ------------------------- //
    _help(command) {
        const optionsList = [];
        [...(command?.options || []), ...this._options].forEach((o) => {
            optionsList.push(`${o.name} ${o.alias || ""}`.trim());
        });
        const optionsPadLength = (0, utilities_js_1.maxLength)(optionsList) + 2;
        optionsList.forEach((s, i) => {
            const optionName = s.split(" ")[0];
            const optionMeta = [...(command?.options || []), ...this._options].find((o) => o.name === optionName);
            optionsList[i] = (0, utilities_js_1.padStringTo)(s, optionsPadLength) + (optionMeta?.description || "");
        });
        if (command) {
            const usage = [];
            usage.push(command.name);
            if (command.arguments)
                usage.push(command.arguments);
            const description = (command.description) ? `  ${command.description}\n\n` : "";
            console.log(`Usage: ${usage.join(" ")}\n\n` +
                description +
                `Options:\n  ${optionsList.join("\n  ")}\n`);
        }
        else {
            const commandsList = [];
            this._commands.forEach((c) => {
                commandsList.push(`${c.name} ${c.arguments || ""}`.trim());
            });
            this._groups.forEach((_g, n) => {
                commandsList.push(n);
            });
            const commandsPadLength = (0, utilities_js_1.maxLength)(commandsList) + 2;
            commandsList.forEach((s, i) => {
                const commandName = s.split(" ")[0];
                const commandMeta = this._commands.find((c) => c.name === commandName);
                const groupMeta = this._groups.get(commandName);
                const description = commandMeta?.description || groupMeta?._description || "";
                commandsList[i] = (0, utilities_js_1.padStringTo)(s, commandsPadLength) + description;
            });
            const description = (this._description) ? `${this._description}\n\n` : "";
            console.log(description +
                `Options:\n  ${optionsList.join("\n  ")}\n\n` +
                `Commands:\n  ${commandsList.join("\n  ")}\n`);
        }
    }
    _run(input = dntShim.Deno.args) {
        const command = this._commands.find((c) => c.name === input[0]);
        const { flags, options } = (0, parsing_js_1.parseOptions)(input, [...this._options, ...(command?.options || [])]);
        // A hack to remove all options from input, due to a
        // limitation with iterating over the array instead
        const providedArgs = (command ? input.slice(1) : input).join(" ").replace(regexes_js_1.default.optionParse, "").split(" ");
        const args = (0, parsing_js_1.parseArguments)(command?.arguments || "", providedArgs);
        // Middleware for the command action
        // This should be the last middleware, so next is ignored
        const actionMiddleware = (ctx, _next) => {
            if (!command) {
                const group = this._groups.get(input[0]);
                if (!group) {
                    this._help();
                    return;
                }
                group._run(input.slice(1));
                return;
            }
            if (flags?.help) {
                this._help(command);
                return;
            }
            command.action(ctx);
        };
        this.use(actionMiddleware);
        this._runMiddleware({ arguments: args, flags, options });
    }
    _runMiddleware(context) {
        const next = () => {
            const middleware = this._middlewares.shift();
            if (middleware)
                middleware(context, next);
        };
        next();
    }
    _validateCommand = (command) => {
        if (command.name.length <= 0) {
            (0, logging_js_1.error)("Command names must be at least 1 character");
        }
        if (!regexes_js_1.default.commandValidate.exec(command.name)) {
            (0, logging_js_1.error)("Command name has invalid characters");
        }
        if (this._commands.find((e) => e.name === command.name)) {
            (0, logging_js_1.error)("Command ${command.name} already exists");
        }
        if (command.options)
            this._validateOptions(command.options);
        if (command.arguments) {
            if (!regexes_js_1.default.argumentParse.test(command.arguments)) {
                (0, logging_js_1.error)(`Arguments for command ${command.name} are formatted incorrectly`);
            }
            if (command.arguments.includes("__proto__")) {
                (0, logging_js_1.error)(`Arguments cannot be named "__proto__"`);
            }
        }
    };
    _validateGroup = (name, group) => {
        if (name.length <= 0)
            (0, logging_js_1.error)("Group names must be at least 1 character");
        if (!regexes_js_1.default.commandValidate.exec(name)) {
            (0, logging_js_1.error)("Group name has invalid characters");
        }
        if (group._commands.length <= 0) {
            (0, logging_js_1.error)("Groups must contain at least 1 command");
        }
        if (group === this)
            (0, logging_js_1.error)("Cannot make a group its own child");
    };
    _validateOptions = (optionArray) => {
        for (const option of optionArray) {
            if (this._options.find((e) => e.name === option.name)) {
                (0, logging_js_1.error)(`Option ${option.name} already exists in global options`);
            }
            if (!regexes_js_1.default.optionValidate.test(option.name)) {
                (0, logging_js_1.error)(` "${option.name}" has an incorrectly formatted name`);
            }
            if (!option.alias)
                continue;
            if (this._options.find((e) => e.alias === option.alias)) {
                (0, logging_js_1.error)(`Option ${option.name} already exists in global options, or it's alias is already in use`);
            }
            if (!regexes_js_1.default.aliasValidate.test(option.alias)) {
                (0, logging_js_1.error)(`Alias "${option.alias}" for option ${option.name} is formatted incorrectly`);
            }
            if (option.name === "__proto__") {
                (0, logging_js_1.error)(`Option "${option.name}" has illegal name`);
            }
        }
    };
    // ---------------------------- Constructor ---------------------------- //
    constructor(spec) {
        this._commands = [];
        this._options = [];
        this._groups = new Map();
        this._description = spec?.description || "";
        this._middlewares = [];
        if (spec?.options) {
            this._validateOptions(spec.options);
            this._options.push(...spec.options);
        }
    }
    // --------------------------- Public Methods --------------------------- //
    command(command) {
        this._validateCommand(command);
        this._commands.push(command);
        return this;
    }
    commands(...commands) {
        commands.forEach((c) => {
            this.command(c);
        });
        return this;
    }
    group(name, group) {
        this._validateGroup(name, group);
        group._options.push(...this._options);
        this._groups.set(name, group);
        return this;
    }
    use(middleware) {
        this._middlewares.push(middleware);
        return this;
    }
}
exports.Group = Group;
